@@ -16,7 +16,7 @@ from scipy.optimize import minimize
 from scipy.special import factorial
 
 from neuror.cut_plane.planes import HalfSpace, PlaneEquation
-from neuror.cut_plane.legacy_detection import cut_detect
+from neuror.cut_plane.legacy_detection import internal_cut_detection
 
 L = logging.getLogger(__name__)
 
@@ -168,14 +168,14 @@ class CutPlane(HalfSpace):
         if not isinstance(neuron, Neuron):
             neuron = load_neuron(neuron)
 
-        cut_leaves, side = cut_detect(neuron, axis)
+        cut_leaves, side = internal_cut_detection(neuron, axis)
 
         plane = cls([int(axis.upper() == 'X'), int(axis.upper() == 'Y'),
                      int(axis.upper() == 'Z'), 0],
                     upward=(side < 0),
-                    morphology=neuron,
+                    morphology=None,
                     bin_width=0)
-
+        plane.morphology = load_neuron(neuron)
         plane.cut_leaves_coordinates = cut_leaves
         return plane
 
