@@ -7,7 +7,7 @@ from collections import Counter, OrderedDict, defaultdict
 from enum import Enum
 from itertools import chain
 from pathlib import Path
-from typing import Any, Optional
+from typing import Any, Dict, Optional
 from nptyping import NDArray
 
 import neurom as nm
@@ -22,7 +22,7 @@ from scipy.spatial.distance import cdist
 
 from neuror import axon
 from neuror.cut_plane import CutPlane
-from neuror.utils import direction, rotation_matrix, section_length
+from neuror.utils import direction, rotation_matrix, section_length, repair_type_map, RepairType
 
 SEG_LENGTH = 5.0
 SHOLL_LAYER_SIZE = 10
@@ -45,19 +45,6 @@ class Action(Enum):
     TERMINATION = 3
 
 
-class RepairType(Enum):
-    '''The types used for the repair.
-
-    based on
-    https://bbpcode.epfl.ch/browse/code/platform/BlueRepairSDK/tree/BlueRepairSDK/src/helper_dendrite.h#n22
-    '''
-    trunk = 0
-    tuft = 1
-    oblique = 2
-    basal = 3
-    axon = 4
-
-
 def is_cut_section(section, cut_points):
     '''Return true if the section is close from the cut plane'''
     if cut_points.size == 0 or section.points.size == 0:
@@ -256,15 +243,19 @@ class Repair(object):
                  axons: Optional[Path] = None,
                  seed: Optional[int] = 0,
                  cut_leaves_coordinates: Optional[NDArray[(3, Any)]] = None,
-                 legacy_detection: bool = False):
+                 legacy_detection: bool = False,
+                 repair_flags: Optional[Dict[RepairType, bool]] = None):
         '''Repair the input morphology
 
         Args:
             inputfile: the input neuron to repair
             axons: donor axons whose section will be used to repair this axon
             seed: the numpy seed
+            cut_leaves_coordinates: List of 3D coordinates from which to start the repair
             legacy_detection: if True, use the legacy cut plane detection
                 (see neuror.legacy_detection)
+            repair_flags: a dict of flags where key is a RepairType and value is whether
+                it should be repaired or not. If not provided, all types will be repaired.
 
         Note: based on https://bbpcode.epfl.ch/browse/code/platform/BlueRepairSDK/tree/BlueRepairSDK/src/repair.cpp#n469  # noqa, pylint: disable=line-too-long
         '''
@@ -273,6 +264,7 @@ class Repair(object):
         self.inputfile = inputfile
         self.axon_donors = axons or list()
         self.donated_intact_axon_sections = list()
+        self.repair_flags = repair_flags or dict()
 
         if legacy_detection:
             self.cut_leaves = CutPlane.find_legacy(inputfile, 'z').cut_leaves_coordinates
@@ -294,7 +286,7 @@ class Repair(object):
         else:
             self.apical_section = None
 
-    # pylint: disable=too-many-locals
+    # pylint: disable=too-many-locals, too-many-branches
     def run(self,
             outputfile: Path,
             plot_file: Optional[Path] = None):
@@ -313,11 +305,10 @@ class Repair(object):
             else:
                 plane = CutPlane.find(axon_donor)
             keep_axons_alive.append(plane)
-            no_cut_plane = (plane.minus_log_prob < 50)
             self.donated_intact_axon_sections.extend(
                 [section for section in iter_sections(plane.morphology)
                  if section.type == SectionType.axon and
-                 (no_cut_plane or is_branch_intact(section, plane.cut_leaves_coordinates))])
+                 is_branch_intact(section, plane.cut_leaves_coordinates)])
 
         self._fill_repair_type_map()
         self._fill_statistics_for_intact_subtrees()
@@ -343,6 +334,8 @@ class Repair(object):
 
         for section in sorted(cut_sections_in_bounding_cylinder, key=section_path_length):
             type_ = self.repair_type_map[section]
+            if not self.repair_flags.get(type_, True):
+                continue
             L.info('Repairing: %s, section id: %s', type_, section.id)
             if type_ in {RepairType.basal, RepairType.oblique, RepairType.tuft}:
                 origin = self._get_origin(section)
@@ -399,7 +392,7 @@ class Repair(object):
             L.warning("No intact basals found. Falling back on less strict selection.")
             basals = [section for section in iter_sections(self.neuron)
                       if (section.type == NeuriteType.basal_dendrite and
-                          is_cut_section(section, self.cut_leaves))]
+                          not is_cut_section(section, self.cut_leaves))]
 
         axons = [neurite.root_node for neurite in iter_neurites(self.neuron)
                  if (neurite.type == NeuriteType.axon and
@@ -504,13 +497,24 @@ class Repair(object):
                     section.points[[0, -1], COLS.XYZ] - origin, axis=1) // SHOLL_LAYER_SIZE).astype(
                         int)
                 per_type = data[repair_type]
-                for layer in range(min(first_layer, last_layer), max(first_layer, last_layer) + 1):
+
+                starting_layer = min(first_layer, last_layer)
+
+                # TODO: why starting_layer + 1 and not starting_layer ?
+                # bcoste The continuation from the starting layer should be taken into account
+                # But that's how it is done in:
+                # https://bbpcode.epfl.ch/source/xref/platform/BlueRepairSDK/BlueRepairSDK/src/morphstats.cpp#88
+                for layer in range(starting_layer + 1, max(first_layer, last_layer)):
                     if order not in per_type[layer]:
                         per_type[layer][order] = {Action.TERMINATION: 0,
                                                   Action.CONTINUATION: 0,
                                                   Action.BIFURCATION: 0}
                     per_type[layer][order][Action.CONTINUATION] += 1
 
+                if order not in per_type[last_layer]:
+                    per_type[last_layer][order] = {Action.TERMINATION: 0,
+                                                   Action.CONTINUATION: 0,
+                                                   Action.BIFURCATION: 0}
                 per_type[last_layer][order][Action.BIFURCATION if section.children else
                                             Action.TERMINATION] += 1
         return data
@@ -606,21 +610,7 @@ class Repair(object):
 
         Note: based on https://bbpcode.epfl.ch/browse/code/platform/BlueRepairSDK/tree/BlueRepairSDK/src/repair.cpp#n242  # noqa, pylint: disable=line-too-long
         '''
-        for section in iter_sections(self.neuron):
-            if section.type == SectionType.apical_dendrite:
-                self.repair_type_map[section] = RepairType.oblique
-            elif section.type == SectionType.basal_dendrite:
-                self.repair_type_map[section] = RepairType.basal
-            elif section.type == SectionType.axon:
-                self.repair_type_map[section] = RepairType.axon
-
-        if self.apical_section is not None:
-            for section in self.apical_section.ipreorder():
-                self.repair_type_map[section] = RepairType.tuft
-
-            # The value for the apical section must be overriden to 'trunk'
-            for section in self.apical_section.iupstream():
-                self.repair_type_map[section] = RepairType.trunk
+        self.repair_type_map = repair_type_map(self.neuron, self.apical_section)
 
 
 def repair(inputfile: Path,
@@ -629,7 +619,8 @@ def repair(inputfile: Path,
            seed: int = 0,
            cut_leaves_coordinates: Optional[NDArray[(3, Any)]] = None,
            legacy_detection: bool = False,
-           plot_file: Optional[Path] = None):
+           plot_file: Optional[Path] = None,
+           repair_flags: Optional[Dict[RepairType, bool]] = None):
     '''The repair function
 
     Args:
@@ -638,6 +629,9 @@ def repair(inputfile: Path,
         axons: the axons
         seed: the numpy seed
         cut_leaves_coordinates: List of 3D coordinates from which to start the repair
+        plot_file: the filename of the plot
+        repair_flags: a dict of flags where key is a RepairType and value is whether
+            it should be repaired or not. If not provided, all types will be repaired.
     '''
     ignored_warnings = (
         # We append the section at the wrong place and then we reposition them
@@ -657,7 +651,7 @@ def repair(inputfile: Path,
     if axons is None:
         axons = list()
     obj = Repair(inputfile, axons=axons, seed=seed, cut_leaves_coordinates=cut_leaves_coordinates,
-                 legacy_detection=legacy_detection)
+                 legacy_detection=legacy_detection, repair_flags=repair_flags)
     obj.run(outputfile, plot_file=plot_file)
 
     for warning in ignored_warnings:
